@@ -1,3 +1,5 @@
+# This file is a part of Julia. License is MIT: http://julialang.org/license
+
 module Git
 #
 # some utility functions for working with git repos
@@ -19,7 +21,7 @@ function git(d)
 end
 
 cmd(args::Cmd; dir="") = `$(git(dir)) $args`
-run(args::Cmd; dir="", out=STDOUT) = Base.run(cmd(args,dir=dir) |> out)
+run(args::Cmd; dir="", out=STDOUT) = Base.run(pipeline(cmd(args,dir=dir), out))
 readall(args::Cmd; dir="") = Base.readall(cmd(args,dir=dir))
 readchomp(args::Cmd; dir="") = Base.readchomp(cmd(args,dir=dir))
 
@@ -34,14 +36,14 @@ function version()
     vs = split(readchomp(`version`), ' ')[3]
     ns = split(vs, '.')
     if length(ns) > 3
-        convert(VersionNumber, join(ns[1:3], '.'))
+        VersionNumber(join(ns[1:3], '.'))
     else
-        convert(VersionNumber, join(ns, '.'))
+        VersionNumber(join(ns, '.'))
     end
 end
 
 modules(args::Cmd; dir="") = readchomp(`config -f .gitmodules $args`, dir=dir)
-different(verA::String, verB::String, path::String; dir="") =
+different(verA::AbstractString, verB::AbstractString, path::AbstractString; dir="") =
     !success(`diff-tree --quiet $verA $verB -- $path`, dir=dir)
 
 dirty(; dir="") = !success(`diff-index --quiet HEAD`, dir=dir)
@@ -98,7 +100,7 @@ function transact(f::Function; dir="")
     end
 end
 
-function is_ancestor_of(a::String, b::String; dir="")
+function is_ancestor_of(a::AbstractString, b::AbstractString; dir="")
     A = readchomp(`rev-parse $a`, dir=dir)
     readchomp(`merge-base $A $b`, dir=dir) == A
 end
@@ -106,17 +108,17 @@ end
 const GITHUB_REGEX =
     r"^(?:git@|git://|https://(?:[\w\.\+\-]+@)?)github.com[:/](([^/].+)/(.+?))(?:\.git)?$"i
 
-function set_remote_url(url::String; remote::String="origin", dir="")
+function set_remote_url(url::AbstractString; remote::AbstractString="origin", dir="")
     run(`config remote.$remote.url $url`, dir=dir)
     m = match(GITHUB_REGEX,url)
-    m == nothing && return
+    m === nothing && return
     push = "git@github.com:$(m.captures[1]).git"
     push != url && run(`config remote.$remote.pushurl $push`, dir=dir)
 end
 
-function normalize_url(url::String)
+function normalize_url(url::AbstractString)
     m = match(GITHUB_REGEX,url)
-    m == nothing ? url : "git://github.com/$(m.captures[1]).git"
+    m === nothing ? url : "git://github.com/$(m.captures[1]).git"
 end
 
 end # module
