@@ -1,9 +1,11 @@
+# This file is a part of Julia. License is MIT: http://julialang.org/license
+
 ##### mean #####
 
 function mean(iterable)
     state = start(iterable)
     if done(iterable, state)
-        error("mean of empty collection undefined: $(repr(iterable))")
+        throw(ArgumentError("mean of empty collection undefined: $(repr(iterable))"))
     end
     count = 1
     total, state = next(iterable, state)
@@ -18,158 +20,161 @@ mean(A::AbstractArray) = sum(A) / length(A)
 
 function mean!{T}(R::AbstractArray{T}, A::AbstractArray)
     sum!(R, A; init=true)
-    lenR = length(R)
-    rs = convert(T, length(A) / lenR)
-    if rs != 1
-        for i = 1:lenR
-            @inbounds R[i] /= rs
-        end
-    end
+    scale!(R, length(R) / length(A))
     return R
 end
 
 momenttype{T}(::Type{T}) = typeof((zero(T) + zero(T)) / 2)
 momenttype(::Type{Float32}) = Float32
-momenttype{T<:Union(Float64,Int32,Int64,Uint32,Uint64)}(::Type{T}) = Float64
+momenttype{T<:Union{Float64,Int32,Int64,UInt32,UInt64}}(::Type{T}) = Float64
 
 mean{T}(A::AbstractArray{T}, region) =
-    mean!(Array(momenttype(T), reduced_dims(size(A), region)), A)
+    mean!(reducedim_initarray(A, region, 0, momenttype(T)), A)
 
 
 ##### variances #####
 
+# faster computation of real(conj(x)*y)
+realXcY(x::Real, y::Real) = x*y
+realXcY(x::Complex, y::Complex) = real(x)*real(y) + imag(x)*imag(y)
+
 function var(iterable; corrected::Bool=true, mean=nothing)
     state = start(iterable)
     if done(iterable, state)
-        error("variance of empty collection undefined: $(repr(iterable))")
+        throw(ArgumentError("variance of empty collection undefined: $(repr(iterable))"))
     end
     count = 1
     value, state = next(iterable, state)
-    if mean == nothing
+    if mean === nothing
         # Use Welford algorithm as seen in (among other places)
         # Knuth's TAOCP, Vol 2, page 232, 3rd edition.
         M = value / 1
-        S = zero(M)
+        S = real(zero(M))
         while !done(iterable, state)
             value, state = next(iterable, state)
             count += 1
             new_M = M + (value - M) / count
-            S = S + (value - M) * (value - new_M)
+            S = S + realXcY(value - M, value - new_M)
             M = new_M
         end
-        return S / (count - int(corrected))
+        return S / (count - Int(corrected))
     elseif isa(mean, Number) # mean provided
         # Cannot use a compensated version, e.g. the one from
         # "Updating Formulae and a Pairwise Algorithm for Computing Sample Variances."
         # by Chan, Golub, and LeVeque, Technical Report STAN-CS-79-773,
         # Department of Computer Science, Stanford University,
         # because user can provide mean value that is different to mean(iterable)
-        sum2 = (value - mean::Number)^2
+        sum2 = abs2(value - mean::Number)
         while !done(iterable, state)
             value, state = next(iterable, state)
             count += 1
-            sum2 += (value - mean)^2
+            sum2 += abs2(value - mean)
         end
-        return sum2 / (count - int(corrected))
+        return sum2 / (count - Int(corrected))
     else
-        error("invalid value of mean")
+        throw(ArgumentError("invalid value of mean, $(mean)::$(typeof(mean))"))
     end
 end
 
 function varzm{T}(A::AbstractArray{T}; corrected::Bool=true)
     n = length(A)
-    n == 0 && return convert(momenttype(T), NaN)
-    return sumabs2(A) / (n - int(corrected))
+    n == 0 && return convert(real(momenttype(T)), NaN)
+    return sumabs2(A) / (n - Int(corrected))
 end
 
 function varzm!{S}(R::AbstractArray{S}, A::AbstractArray; corrected::Bool=true)
     if isempty(A)
         fill!(R, convert(S, NaN))
     else
-        rn = div(length(A), length(r)) - int(corrected)
+        rn = div(length(A), length(r)) - Int(corrected)
         scale!(sumabs2!(R, A; init=true), convert(S, 1/rn))
     end
     return R
 end
 
 varzm{T}(A::AbstractArray{T}, region; corrected::Bool=true) =
-    varzm!(Array(momenttype(T), reduced_dims(A, region)), A; corrected=corrected)
+    varzm!(reducedim_initarray(A, region, 0, real(momenttype(T))), A; corrected=corrected)
 
 immutable CentralizedAbs2Fun{T<:Number} <: Func{1}
     m::T
 end
-evaluate(f::CentralizedAbs2Fun, x) = abs2(x - f.m)
+call(f::CentralizedAbs2Fun, x) = abs2(x - f.m)
+centralize_sumabs2(A::AbstractArray, m::Number) =
+    mapreduce(CentralizedAbs2Fun(m), AddFun(), A)
 centralize_sumabs2(A::AbstractArray, m::Number, ifirst::Int, ilast::Int) =
     mapreduce_impl(CentralizedAbs2Fun(m), AddFun(), A, ifirst, ilast)
 
-@ngenerate N typeof(R) function centralize_sumabs2!{S,T,N}(R::AbstractArray{S}, A::AbstractArray{T,N}, means::AbstractArray)
-    # following the implementation of _mapreducedim! at base/reducedim.jl
-    lsiz = check_reducdims(R, A)
-    isempty(R) || fill!(R, zero(S))
-    isempty(A) && return R
-    @nextract N sizeR d->size(R,d)
-    sizA1 = size(A, 1)
+@generated function centralize_sumabs2!{S,T,N}(R::AbstractArray{S}, A::AbstractArray{T,N}, means::AbstractArray)
+    quote
+        # following the implementation of _mapreducedim! at base/reducedim.jl
+        lsiz = check_reducedims(R,A)
+        isempty(R) || fill!(R, zero(S))
+        isempty(A) && return R
+        @nextract $N sizeR d->size(R,d)
+        sizA1 = size(A, 1)
 
-    if has_fast_linear_indexing(A) && lsiz > 16
-        # use centralize_sumabs2, which is probably better tuned to achieve higher performance
-        nslices = div(length(A), lsiz)
-        ibase = 0
-        for i = 1:nslices
-            @inbounds R[i] = centralize_sumabs2(A, means[i], ibase+1, ibase+lsiz)
-            ibase += lsiz
-        end
-    elseif size(R, 1) == 1 && sizA1 > 1
-        # keep the accumulator as a local variable when reducing along the first dimension
-        @nloops N i d->(d>1? (1:size(A,d)) : (1:1)) d->(j_d = sizeR_d==1 ? 1 : i_d) begin
-            @inbounds r = (@nref N R j)
-            @inbounds m = (@nref N means j)
-            for i_1 = 1:sizA1
-                @inbounds r += abs2((@nref N A i) - m)
+        if has_fast_linear_indexing(A) && lsiz > 16
+            # use centralize_sumabs2, which is probably better tuned to achieve higher performance
+            nslices = div(length(A), lsiz)
+            ibase = 0
+            for i = 1:nslices
+                @inbounds R[i] = centralize_sumabs2(A, means[i], ibase+1, ibase+lsiz)
+                ibase += lsiz
+            end
+        elseif size(R, 1) == 1 && sizA1 > 1
+            # keep the accumulator as a local variable when reducing along the first dimension
+            @nloops $N i d->(d>1? (1:size(A,d)) : (1:1)) d->(j_d = sizeR_d==1 ? 1 : i_d) begin
+                @inbounds r = (@nref $N R j)
+                @inbounds m = (@nref $N means j)
+                for i_1 = 1:sizA1
+                    @inbounds r += abs2((@nref $N A i) - m)
+                end
+                @inbounds (@nref $N R j) = r
+            end
+        else
+            # general implementation
+            @nloops $N i A d->(j_d = sizeR_d==1 ? 1 : i_d) begin
+                @inbounds (@nref $N R j) += abs2((@nref $N A i) - (@nref $N means j))
             end
-            @inbounds (@nref N R j) = r
-        end
-    else
-        # general implementation
-        @nloops N i A d->(j_d = sizeR_d==1 ? 1 : i_d) begin
-            @inbounds (@nref N R j) += abs2((@nref N A i) - (@nref N means j))
         end
+        return R
     end
-    return R
 end
 
 function varm{T}(A::AbstractArray{T}, m::Number; corrected::Bool=true)
     n = length(A)
-    n == 0 && return convert(momenttype(T), NaN)
-    n == 1 && return convert(momenttype(T), abs2(A[1] - m)/(1 - int(corrected)))
-    return centralize_sumabs2(A, m, 1, n) / (n - int(corrected))
+    n == 0 && return convert(real(momenttype(T)), NaN)
+    n == 1 && return convert(real(momenttype(T)), abs2(A[1] - m)/(1 - Int(corrected)))
+    return centralize_sumabs2(A, m) / (n - Int(corrected))
 end
 
 function varm!{S}(R::AbstractArray{S}, A::AbstractArray, m::AbstractArray; corrected::Bool=true)
     if isempty(A)
         fill!(R, convert(S, NaN))
     else
-        rn = div(length(A), length(R)) - int(corrected)
+        rn = div(length(A), length(R)) - Int(corrected)
         scale!(centralize_sumabs2!(R, A, m), convert(S, 1/rn))
     end
     return R
 end
 
 varm{T}(A::AbstractArray{T}, m::AbstractArray, region; corrected::Bool=true) =
-    varm!(Array(momenttype(T), reduced_dims(size(A), region)), A, m; corrected=corrected)
+    varm!(reducedim_initarray(A, region, 0, real(momenttype(T))), A, m; corrected=corrected)
 
 
 function var{T}(A::AbstractArray{T}; corrected::Bool=true, mean=nothing)
-    convert(momenttype(T), mean == 0 ? varzm(A; corrected=corrected) :
-                           mean == nothing ? varm(A, Base.mean(A); corrected=corrected) :
-                           isa(mean, Number) ? varm(A, mean::Number; corrected=corrected) :
-                           error("invalid value of mean"))::momenttype(T)
+    convert(real(momenttype(T)),
+            mean == 0 ? varzm(A; corrected=corrected) :
+            mean === nothing ? varm(A, Base.mean(A); corrected=corrected) :
+            isa(mean, Number) ? varm(A, mean::Number; corrected=corrected) :
+            throw(ArgumentError("invalid value of mean, $(mean)::$(typeof(mean))")))::real(momenttype(T))
 end
 
 function var(A::AbstractArray, region; corrected::Bool=true, mean=nothing)
     mean == 0 ? varzm(A, region; corrected=corrected) :
-    mean == nothing ? varm(A, Base.mean(A, region), region; corrected=corrected) :
+    mean === nothing ? varm(A, Base.mean(A, region), region; corrected=corrected) :
     isa(mean, AbstractArray) ? varm(A, mean::AbstractArray, region; corrected=corrected) :
-    error("invalid value of mean")
+    throw(ArgumentError("invalid value of mean, $(mean)::$(typeof(mean))"))
 end
 
 varm(iterable, m::Number; corrected::Bool=true) =
@@ -177,7 +182,15 @@ varm(iterable, m::Number; corrected::Bool=true) =
 
 ## variances over ranges
 
-varm(v::Range, m::Number) = var(v)
+function varm(v::Range, m::Number)
+    f = first(v) - m
+    s = step(v)
+    l = length(v)
+    if l == 0 || l == 1
+           return NaN
+    end
+    return f^2 * l / (l - 1) + f * s * l + s^2 * l * (2 * l - 1) / 6
+end
 
 function var(v::Range)
     s = step(v)
@@ -192,7 +205,7 @@ end
 ##### standard deviation #####
 
 function sqrt!(A::AbstractArray)
-    for i = 1:length(A)
+    for i in eachindex(A)
         @inbounds A[i] = sqrt(A[i])
     end
     A
@@ -226,7 +239,7 @@ _getnobs(x::AbstractMatrix, vardim::Int) = size(x, vardim)
 
 function _getnobs(x::AbstractVecOrMat, y::AbstractVecOrMat, vardim::Int)
     n = _getnobs(x, vardim)
-    _getnobs(y, vardim) == n || throw(DimensionMismatch("Dimensions of x and y mismatch."))
+    _getnobs(y, vardim) == n || throw(DimensionMismatch("dimensions of x and y mismatch"))
     return n
 end
 
@@ -235,7 +248,7 @@ _vmean(x::AbstractMatrix, vardim::Int) = mean(x, vardim)
 
 # core functions
 
-unscaled_covzm(x::AbstractVector) = dot(x, x)
+unscaled_covzm(x::AbstractVector) = sumabs2(x)
 unscaled_covzm(x::AbstractMatrix, vardim::Int) = (vardim == 1 ? _conj(x'x) : x * x')
 
 unscaled_covzm(x::AbstractVector, y::AbstractVector) = dot(x, y)
@@ -248,16 +261,16 @@ unscaled_covzm(x::AbstractMatrix, y::AbstractMatrix, vardim::Int) =
 
 # covzm (with centered data)
 
-covzm(x::AbstractVector; corrected::Bool=true) = unscaled_covzm(x, x) / (length(x) - int(corrected))
+covzm(x::AbstractVector; corrected::Bool=true) = unscaled_covzm(x) / (length(x) - Int(corrected))
 
 covzm(x::AbstractMatrix; vardim::Int=1, corrected::Bool=true) =
-    scale!(unscaled_covzm(x, vardim), inv(size(x,vardim) - int(corrected)))
+    scale!(unscaled_covzm(x, vardim), inv(size(x,vardim) - Int(corrected)))
 
 covzm(x::AbstractVector, y::AbstractVector; corrected::Bool=true) =
-    unscaled_covzm(x, y) / (length(x) - int(corrected))
+    unscaled_covzm(x, y) / (length(x) - Int(corrected))
 
 covzm(x::AbstractVecOrMat, y::AbstractVecOrMat; vardim::Int=1, corrected::Bool=true) =
-    scale!(unscaled_covzm(x, y, vardim), inv(_getnobs(x, y, vardim) - int(corrected)))
+    scale!(unscaled_covzm(x, y, vardim), inv(_getnobs(x, y, vardim) - Int(corrected)))
 
 # covm (with provided mean)
 
@@ -277,34 +290,34 @@ covm(x::AbstractVecOrMat, xmean, y::AbstractVecOrMat, ymean; vardim::Int=1, corr
 
 function cov(x::AbstractVector; corrected::Bool=true, mean=nothing)
     mean == 0 ? covzm(x; corrected=corrected) :
-    mean == nothing ? covm(x, Base.mean(x); corrected=corrected) :
+    mean === nothing ? covm(x, Base.mean(x); corrected=corrected) :
     isa(mean, Number) ? covm(x, mean; corrected=corrected) :
-    error("Invalid value of mean.")
+    throw(ArgumentError("invalid value of mean, $(mean)::$(typeof(mean))"))
 end
 
 function cov(x::AbstractMatrix; vardim::Int=1, corrected::Bool=true, mean=nothing)
     mean == 0 ? covzm(x; vardim=vardim, corrected=corrected) :
-    mean == nothing ? covm(x, _vmean(x, vardim); vardim=vardim, corrected=corrected) :
+    mean === nothing ? covm(x, _vmean(x, vardim); vardim=vardim, corrected=corrected) :
     isa(mean, AbstractArray) ? covm(x, mean; vardim=vardim, corrected=corrected) :
-    error("Invalid value of mean.")
+    throw(ArgumentError("invalid value of mean, $(mean)::$(typeof(mean))"))
 end
 
 function cov(x::AbstractVector, y::AbstractVector; corrected::Bool=true, mean=nothing)
     mean == 0 ? covzm(x, y; corrected=corrected) :
-    mean == nothing ? covm(x, Base.mean(x), y, Base.mean(y); corrected=corrected) :
+    mean === nothing ? covm(x, Base.mean(x), y, Base.mean(y); corrected=corrected) :
     isa(mean, (Number,Number)) ? covm(x, mean[1], y, mean[2]; corrected=corrected) :
-    error("Invalid value of mean.")
+    throw(ArgumentError("invalid value of mean, $(mean)::$(typeof(mean))"))
 end
 
 function cov(x::AbstractVecOrMat, y::AbstractVecOrMat; vardim::Int=1, corrected::Bool=true, mean=nothing)
     if mean == 0
         covzm(x, y; vardim=vardim, corrected=corrected)
-    elseif mean == nothing
+    elseif mean === nothing
         covm(x, _vmean(x, vardim), y, _vmean(y, vardim); vardim=vardim, corrected=corrected)
     elseif isa(mean, (Any,Any))
         covm(x, mean[1], y, mean[2]; vardim=vardim, corrected=corrected)
     else
-        error("Invalid value of mean.")
+        throw(ArgumentError("invalid value of mean, $(mean)::$(typeof(mean))"))
     end
 end
 
@@ -315,7 +328,7 @@ end
 
 function cov2cor!{T}(C::AbstractMatrix{T}, xsd::AbstractArray)
     nx = length(xsd)
-    size(C) == (nx, nx) || throw(DimensionMismatch("Inconsistent dimensions."))
+    size(C) == (nx, nx) || throw(DimensionMismatch("inconsistent dimensions"))
     for j = 1:nx
         for i = 1:j-1
             C[i,j] = C[j,i]
@@ -330,7 +343,7 @@ end
 
 function cov2cor!(C::AbstractMatrix, xsd::Number, ysd::AbstractArray)
     nx, ny = size(C)
-    length(ysd) == ny || throw(DimensionMismatch("Inconsistent dimensions."))
+    length(ysd) == ny || throw(DimensionMismatch("inconsistent dimensions"))
     for j = 1:ny
         for i = 1:nx
             C[i,j] /= (xsd * ysd[j])
@@ -341,7 +354,7 @@ end
 
 function cov2cor!(C::AbstractMatrix, xsd::AbstractArray, ysd::Number)
     nx, ny = size(C)
-    length(xsd) == nx || throw(DimensionMismatch("Inconsistent dimensions."))
+    length(xsd) == nx || throw(DimensionMismatch("inconsistent dimensions"))
     for j = 1:ny
         for i = 1:nx
             C[i,j] /= (xsd[i] * ysd)
@@ -353,7 +366,7 @@ end
 function cov2cor!(C::AbstractMatrix, xsd::AbstractArray, ysd::AbstractArray)
     nx, ny = size(C)
     (length(xsd) == nx && length(ysd) == ny) ||
-        throw(DimensionMismatch("Inconsistent dimensions."))
+        throw(DimensionMismatch("inconsistent dimensions"))
     for j = 1:ny
         for i = 1:nx
             C[i,j] /= (xsd[i] * ysd[j])
@@ -364,14 +377,14 @@ end
 
 # corzm (non-exported, with centered data)
 
-corzm{T}(x::AbstractVector{T}) = float(one(T) * one(T))
+corzm{T}(x::AbstractVector{T}) = one(real(T))
 
 corzm(x::AbstractMatrix; vardim::Int=1) =
     (c = unscaled_covzm(x, vardim); cov2cor!(c, sqrt!(diag(c))))
 
 function corzm(x::AbstractVector, y::AbstractVector)
     n = length(x)
-    length(y) == n || throw(DimensionMismatch("Inconsistent lengths."))
+    length(y) == n || throw(DimensionMismatch("inconsistent lengths"))
     x1 = x[1]
     y1 = y[1]
     xx = abs2(x1)
@@ -400,7 +413,7 @@ corzm(x::AbstractMatrix, y::AbstractMatrix; vardim::Int=1) =
 
 # corm
 
-corm(x::AbstractVector, xmean) = corzm(x .- xmean)
+corm{T}(x::AbstractVector{T}, xmean) = one(real(T))
 
 corm(x::AbstractMatrix, xmean; vardim::Int=1) = corzm(x .- xmean; vardim=vardim)
 
@@ -411,55 +424,73 @@ corm(x::AbstractVecOrMat, xmean, y::AbstractVecOrMat, ymean; vardim::Int=1) =
 
 # cor
 
-function cor(x::AbstractVector; mean=nothing)
-    mean == 0 ? corzm(x) :
-    mean == nothing ? corm(x, Base.mean(x)) :
-    isa(mean, Number) ? corm(x, mean) :
-    error("Invalid value of mean.")
-end
+cor{T}(x::AbstractVector{T}; mean=nothing) = one(real(T))
 
 function cor(x::AbstractMatrix; vardim::Int=1, mean=nothing)
     mean == 0 ? corzm(x; vardim=vardim) :
-    mean == nothing ? corm(x, _vmean(x, vardim); vardim=vardim) :
+    mean === nothing ? corm(x, _vmean(x, vardim); vardim=vardim) :
     isa(mean, AbstractArray) ? corm(x, mean; vardim=vardim) :
-    error("Invalid value of mean.")
+    throw(ArgumentError("invalid value of mean, $(mean)::$(typeof(mean))"))
 end
 
 function cor(x::AbstractVector, y::AbstractVector; mean=nothing)
     mean == 0 ? corzm(x, y) :
-    mean == nothing ? corm(x, Base.mean(x), y, Base.mean(y)) :
+    mean === nothing ? corm(x, Base.mean(x), y, Base.mean(y)) :
     isa(mean, (Number,Number)) ? corm(x, mean[1], y, mean[2]) :
-    error("Invalid value of mean.")
+    throw(ArgumentError("invalid value of mean, $(mean)::$(typeof(mean))"))
 end
 
 function cor(x::AbstractVecOrMat, y::AbstractVecOrMat; vardim::Int=1, mean=nothing)
     if mean == 0
         corzm(x, y; vardim=vardim)
-    elseif mean == nothing
+    elseif mean === nothing
         corm(x, _vmean(x, vardim), y, _vmean(y, vardim); vardim=vardim)
     elseif isa(mean, (Any,Any))
         corm(x, mean[1], y, mean[2]; vardim=vardim)
     else
-        error("Invalid value of mean.")
+        throw(ArgumentError("invalid value of mean, $(mean)::$(typeof(mean))"))
     end
 end
 
 
 ##### median & quantiles #####
 
+"""
+    middle(x)
+
+Compute the middle of a scalar value, which is equivalent to `x` itself, but of the type of `middle(x, x)` for consistency.
+"""
 # Specialized functions for real types allow for improved performance
-middle(x::Union(Bool,Int8,Int16,Int32,Int64,Int128,Uint8,Uint16,Uint32,Uint64,Uint128)) = float64(x)
-middle(x::FloatingPoint) = x
-middle(x::Float16) = float32(x)
+middle(x::Union{Bool,Int8,Int16,Int32,Int64,Int128,UInt8,UInt16,UInt32,UInt64,UInt128}) = Float64(x)
+middle(x::AbstractFloat) = x
+middle(x::Float16) = Float32(x)
 middle(x::Real) = (x + zero(x)) / 1
-middle(x::Real, y::Real) = (x + y) / 2
+
+"""
+    middle(x, y)
+
+Compute the middle of two reals `x` and `y`, which is equivalent in both value and type to computing their mean (`(x + y) / 2`).
+"""
+middle(x::Real, y::Real) = x/2 + y/2
+
+"""
+    middle(range)
+
+Compute the middle of a range, which consists in computing the mean of its extrema. Since a range is sorted, the mean is performed with the first and last element.
+"""
 middle(a::Range) = middle(a[1], a[end])
+
+"""
+    middle(array)
+
+Compute the middle of an array, which consists in finding its extrema and then computing their mean.
+"""
 middle(a::AbstractArray) = ((v1, v2) = extrema(a); middle(v1, v2))
 
-function median!{T}(v::AbstractVector{T}; checknan::Bool=true)
-    isempty(v) && error("median of an empty array is undefined")
-    if checknan && T<:FloatingPoint
-        for x in v
+function median!{T}(v::AbstractVector{T})
+    isempty(v) && throw(ArgumentError("median of an empty array is undefined, $(repr(v))"))
+    if T<:AbstractFloat
+        @inbounds for x in v
             isnan(x) && return x
         end
     end
@@ -471,56 +502,126 @@ function median!{T}(v::AbstractVector{T}; checknan::Bool=true)
         return middle(m[1], m[2])
     end
 end
+median!{T}(v::AbstractArray{T}) = median!(vec(v))
 
-median{T}(v::AbstractArray{T}; checknan::Bool=true) =
-    median!(vec(copy(v)), checknan=checknan)
-median{T}(v::AbstractArray{T}, region; checknan::Bool=true) =
-    mapslices( x->median(x; checknan=checknan), v, region )
+median{T}(v::AbstractArray{T}) = median!(vec(copy(v)))
+median{T}(v::AbstractArray{T}, region) = mapslices(median!, v, region)
 
 # for now, use the R/S definition of quantile; may want variants later
 # see ?quantile in R -- this is type 7
-# TODO: need faster implementation (use select!?)
-#
-function quantile!(v::AbstractVector, q::AbstractVector)
-    isempty(v) && error("empty data array")
-    isempty(q) && error("empty quantile array")
+"""
+    quantile!([q, ] v, p; sorted=false)
 
-    # make sure the quantiles are in [0,1]
-    q = bound_quantiles(q)
+Compute the quantile(s) of a vector `v` at the probabilities `p`, with optional output into
+array `q` (if not provided, a new output array is created). The keyword argument `sorted`
+indicates whether `v` can be assumed to be sorted; if `false` (the default), then the
+elements of `v` may be partially sorted.
+
+The elements of `p` should be on the interval [0,1], and `v` should not have any `NaN`
+values.
+
+Quantiles are computed via linear interpolation between the points `((k-1)/(n-1), v[k])`,
+for `k = 1:n` where `n = length(v)`. This corresponds to Definition 7 of Hyndman and Fan
+(1996), and is the same as the R default.
+
+* Hyndman, R.J and Fan, Y. (1996) "Sample Quantiles in Statistical Packages",
+  *The American Statistician*, Vol. 50, No. 4, pp. 361-365
+"""
+function quantile!(q::AbstractArray, v::AbstractVector, p::AbstractArray;
+                   sorted::Bool=false)
+    size(p) == size(q) || throw(DimensionMismatch())
+
+    isempty(v) && throw(ArgumentError("empty data vector"))
 
     lv = length(v)
-    lq = length(q)
+    if !sorted
+        minp, maxp = extrema(p)
+        lo = floor(Int,1+minp*(lv-1))
+        hi = ceil(Int,1+maxp*(lv-1))
 
-    index = 1 .+ (lv-1)*q
-    lo = ifloor(index)
-    hi = iceil(index)
-    sort!(v)
-    isnan(v[end]) && error("quantiles are undefined in presence of NaNs")
-    i = find(index .> lo)
-    r = float(v[lo])
-    h = (index.-lo)[i]
-    r[i] = (1.-h).*r[i] + h.*v[hi[i]]
-    return r
-end
-quantile(v::AbstractVector, q::AbstractVector) = quantile!(copy(v),q)
-quantile(v::AbstractVector, q::Number) = quantile(v,[q])[1]
-
-function bound_quantiles(qs::AbstractVector)
-    epsilon = 100*eps()
-    if (any(qs .< -epsilon) || any(qs .> 1+epsilon))
-        error("quantiles out of [0,1] range")
+        # only need to perform partial sort
+        sort!(v, 1, lv, PartialQuickSort(lo:hi), Base.Sort.Forward)
     end
-    [min(1,max(0,q)) for q = qs]
+    isnan(v[end]) && throw(ArgumentError("quantiles are undefined in presence of NaNs"))
+
+    for i = 1:length(p)
+        @inbounds q[i] = _quantile(v,p[i])
+    end
+    return q
 end
 
+quantile!(v::AbstractVector, p::AbstractArray; sorted::Bool=false) =
+    quantile!(similar(p,float(eltype(v))), v, p; sorted=sorted)
+
+function quantile!(v::AbstractVector, p::Real;
+                   sorted::Bool=false)
+    isempty(v) && throw(ArgumentError("empty data vector"))
+
+    lv = length(v)
+    if !sorted
+        lo = floor(Int,1+p*(lv-1))
+        hi = ceil(Int,1+p*(lv-1))
+
+        # only need to perform partial sort
+        sort!(v, 1, lv, PartialQuickSort(lo:hi), Base.Sort.Forward)
+    end
+    isnan(v[end]) && throw(ArgumentError("quantiles are undefined in presence of NaNs"))
+
+    return _quantile(v,p)
+end
+
+# Core quantile lookup function: assumes `v` sorted
+@inline function _quantile(v::AbstractVector, p::Real)
+    T = float(eltype(v))
+    isnan(p) && return T(NaN)
+
+    lv = length(v)
+    index = 1 + (lv-1)*p
+    1 <= index <= lv || error("input probability out of [0,1] range")
+
+    indlo = floor(index)
+    i = trunc(Int,indlo)
+
+    if index == indlo
+        return T(v[i])
+    else
+        h = T(index - indlo)
+        return (1-h)*T(v[i]) + h*T(v[i+1])
+    end
+end
+
+
+"""
+    quantile(v, p; sorted=false)
+
+Compute the quantile(s) of a vector `v` at a specified probability or vector `p`. The
+keyword argument `sorted` indicates whether `v` can be assumed to be sorted.
+
+The `p` should be on the interval [0,1], and `v` should not have any `NaN` values.
+
+Quantiles are computed via linear interpolation between the points `((k-1)/(n-1), v[k])`,
+for `k = 1:n` where `n = length(v)`. This corresponds to Definition 7 of Hyndman and Fan
+(1996), and is the same as the R default.
+
+* Hyndman, R.J and Fan, Y. (1996) "Sample Quantiles in Statistical Packages",
+  *The American Statistician*, Vol. 50, No. 4, pp. 361-365
+"""
+quantile(v::AbstractVector, p; sorted::Bool=false) =
+    quantile!(sorted ? v : copy!(similar(v),v), p; sorted=sorted)
 
 
 ##### histogram #####
 
 ## nice-valued ranges for histograms
 
-function histrange{T<:FloatingPoint,N}(v::AbstractArray{T,N}, n::Integer)
-    if length(v) == 0
+function histrange{T<:AbstractFloat,N}(v::AbstractArray{T,N}, n::Integer)
+    nv = length(v)
+    if nv == 0 && n < 0
+        throw(ArgumentError("number of bins must be ≥ 0 for an empty array, got $n"))
+    elseif nv > 0 && n < 1
+        throw(ArgumentError("number of bins must be ≥ 1 for a non-empty array, got $n"))
+    end
+    if nv == 0
         return 0.0:1.0:0.0
     end
     lo, hi = extrema(v)
@@ -539,20 +640,29 @@ function histrange{T<:FloatingPoint,N}(v::AbstractArray{T,N}, n::Integer)
         end
     end
     start = step*(ceil(lo/step)-1)
-    nm1 = iceil((hi - start)/step)
+    nm1 = ceil(Int,(hi - start)/step)
     start:step:(start + nm1*step)
 end
 
 function histrange{T<:Integer,N}(v::AbstractArray{T,N}, n::Integer)
-    if length(v) == 0
+    nv = length(v)
+    if nv == 0 && n < 0
+        throw(ArgumentError("number of bins must be ≥ 0 for an empty array, got $n"))
+    elseif nv > 0 && n < 1
+        throw(ArgumentError("number of bins must be ≥ 1 for a non-empty array, got $n"))
+    end
+    if nv == 0
         return 0:1:0
     end
+    if n <= 0
+        throw(ArgumentError("number of bins n=$n must be positive"))
+    end
     lo, hi = extrema(v)
     if hi == lo
         step = 1
     else
         bw = (hi - lo) / n
-        e = 10^max(0,ifloor(log10(bw)))
+        e = 10^max(0,floor(Int,log10(bw)))
         r = bw / e
         if r <= 1
             step = e
@@ -565,7 +675,7 @@ function histrange{T<:Integer,N}(v::AbstractArray{T,N}, n::Integer)
         end
     end
     start = step*(ceil(lo/step)-1)
-    nm1 = iceil((hi - start)/step)
+    nm1 = ceil(Int,(hi - start)/step)
     start:step:(start + nm1*step)
 end
 
@@ -576,12 +686,12 @@ midpoints(v::AbstractVector) = [0.5*(v[i] + v[i+1]) for i in 1:length(v)-1]
 ## hist ##
 function sturges(n)  # Sturges' formula
     n==0 && return one(n)
-    iceil(log2(n))+1
+    ceil(Int,log2(n))+1
 end
 
 function hist!{HT}(h::AbstractArray{HT}, v::AbstractVector, edg::AbstractVector; init::Bool=true)
     n = length(edg) - 1
-    length(h) == n || error("length(h) must equal length(edg) - 1.")
+    length(h) == n || throw(DimensionMismatch("length(histogram) must equal length(edges) - 1"))
     if init
         fill!(h, zero(HT))
     end
@@ -600,7 +710,9 @@ hist(v::AbstractVector) = hist(v,sturges(length(v)))
 
 function hist!{HT}(H::AbstractArray{HT,2}, A::AbstractMatrix, edg::AbstractVector; init::Bool=true)
     m, n = size(A)
-    size(H) == (length(edg)-1, n) || error("Incorrect size of H.")
+    sH = size(H)
+    sE = (length(edg)-1,n)
+    sH == sE || throw(DimensionMismatch("incorrect size of histogram"))
     if init
         fill!(H, zero(HT))
     end
@@ -618,10 +730,10 @@ hist(A::AbstractMatrix) = hist(A,sturges(size(A,1)))
 ## hist2d
 function hist2d!{HT}(H::AbstractArray{HT,2}, v::AbstractMatrix,
                      edg1::AbstractVector, edg2::AbstractVector; init::Bool=true)
-    size(v,2) == 2 || error("hist2d requires an Nx2 matrix.")
+    size(v,2) == 2 || throw(DimensionMismatch("hist2d requires an Nx2 matrix"))
     n = length(edg1) - 1
     m = length(edg2) - 1
-    size(H) == (n, m) || error("Incorrect size of H.")
+    size(H) == (n, m) || throw(DimensionMismatch("incorrect size of histogram"))
     if init
         fill!(H, zero(HT))
     end
@@ -644,4 +756,3 @@ hist2d(v::AbstractMatrix, n1::Integer, n2::Integer) =
     hist2d(v, histrange(sub(v,:,1),n1), histrange(sub(v,:,2),n2))
 hist2d(v::AbstractMatrix, n::Integer) = hist2d(v, n, n)
 hist2d(v::AbstractMatrix) = hist2d(v, sturges(size(v,1)))
-
