@@ -92,17 +92,22 @@ if ! [ -e julia-installer.exe ]; then
   $SEVENZIP x -y $f >> get-deps.log
 fi
 for i in bin/*.dll Git/usr/bin/*.dll Git/usr/bin/*.exe; do
-  $SEVENZIP e -y julia-installer.exe "\$_OUTDIR/$i" \
+  $SEVENZIP e -y julia-installer.exe "$i" \
     -ousr\\`dirname $i | sed -e 's|/julia||' -e 's|/|\\\\|g'` >> get-deps.log
 done
 for i in share/julia/base/pcre_h.jl; do
-  $SEVENZIP e -y julia-installer.exe "\$_OUTDIR/$i" -obase >> get-deps.log
+  $SEVENZIP e -y julia-installer.exe "$i" -obase >> get-deps.log
 done
 # suppress "bash.exe: warning: could not find /tmp, please create!"
 mkdir -p usr/Git/tmp
 # Remove libjulia.dll if it was copied from downloaded binary
 rm -f usr/bin/libjulia.dll
 rm -f usr/bin/libjulia-debug.dll
+rm -f usr/bin/libgcc_s_s*-1.dll
+rm -f usr/bin/libgfortran-3.dll
+rm -f usr/bin/libquadmath-0.dll
+rm -f usr/bin/libssp-0.dll
+rm -f usr/bin/libstdc++-6.dll
 
 if [ -z "$USEMSVC" ]; then
   if [ -z "`which ${CROSS_COMPILE}gcc 2>/dev/null`" ]; then
