@@ -453,7 +453,7 @@ JL_CALLABLE(jl_f_apply)
         newargs[0] = (jl_value_t*)arg_heap;
         newargs = jl_svec_data(arg_heap);
     }
-    // GC Note: here we assume that the the return value of `jl_svecref`,
+    // GC Note: here we assume that the return value of `jl_svecref`,
     //          `jl_cellref` will not be young if `arg_heap` becomes old
     //          since they are allocated before `arg_heap`. Otherwise,
     //          we need to add write barrier for !onstack
@@ -1576,7 +1576,7 @@ static size_t jl_static_show_x_(JL_STREAM *out, jl_value_t *v,
         size_t nb = jl_datatype_size(vt);
         size_t tlen = jl_datatype_nfields(vt);
         if (nb > 0 && tlen == 0) {
-            char *data = (char*)jl_data_ptr(v);
+            uint8_t *data = (uint8_t*)v;
             n += jl_printf(out, "0x");
             for(int i=nb-1; i >= 0; --i)
                 n += jl_printf(out, "%02" PRIx8, data[i]);
