@@ -270,10 +270,7 @@ static region_t *find_region(void *ptr, int maybe);
 
 NOINLINE static uintptr_t gc_get_stack_ptr()
 {
-    void *dummy = NULL;
-    // The mask is to suppress the compiler warning about returning
-    // address of local variable
-    return (uintptr_t)&dummy & ~(uintptr_t)15;
+    return (uintptr_t)jl_get_frame_addr();
 }
 
 #include "gc-debug.c"
