@@ -1,3 +1,5 @@
+// This file is a part of Julia. License is MIT: http://julialang.org/license
+
 #include <stdlib.h>
 #include <setjmp.h>
 #include <assert.h>
@@ -14,21 +16,16 @@ extern "C" {
 
 extern int jl_lineno;
 
-static jl_value_t *eval(jl_value_t *e, jl_value_t **locals, size_t nl);
-static jl_value_t *eval_body(jl_array_t *stmts, jl_value_t **locals, size_t nl,
+static jl_value_t *eval(jl_value_t *e, jl_value_t **locals, size_t nl, size_t ngensym);
+static jl_value_t *eval_body(jl_array_t *stmts, jl_value_t **locals, size_t nl, size_t ngensym,
                              int start, int toplevel);
 jl_value_t *jl_eval_module_expr(jl_expr_t *ex);
 int jl_is_toplevel_only_expr(jl_value_t *e);
+extern void *jl_symbol_name(jl_sym_t *s);
 
 jl_value_t *jl_interpret_toplevel_expr(jl_value_t *e)
 {
-    return eval(e, NULL, 0);
-}
-
-jl_value_t *jl_interpret_toplevel_expr_with(jl_value_t *e,
-                                            jl_value_t **locals, size_t nl)
-{
-    return eval(e, locals, nl);
+    return eval(e, NULL, 0, 0);
 }
 
 DLLEXPORT jl_value_t *jl_interpret_toplevel_expr_in(jl_module_t *m, jl_value_t *e,
@@ -39,7 +36,7 @@ DLLEXPORT jl_value_t *jl_interpret_toplevel_expr_in(jl_module_t *m, jl_value_t *
     jl_module_t *task_last_m = jl_current_task->current_module;
     JL_TRY {
         jl_current_task->current_module = jl_current_module = m;
-        v = eval(e, locals, nl);
+        v = eval(e, locals, nl, 0);
     }
     JL_CATCH {
         jl_current_module = last_m;
@@ -53,15 +50,19 @@ DLLEXPORT jl_value_t *jl_interpret_toplevel_expr_in(jl_module_t *m, jl_value_t *
 }
 
 static jl_value_t *do_call(jl_function_t *f, jl_value_t **args, size_t nargs,
-                           jl_value_t **locals, size_t nl)
+                           jl_value_t *eval0, jl_value_t **locals, size_t nl, size_t ngensym)
 {
     jl_value_t **argv;
     JL_GC_PUSHARGS(argv, nargs+1);
     size_t i;
     argv[0] = (jl_value_t*)f;
-    for(i=1; i < nargs+1; i++) argv[i] = NULL;
-    for(i=0; i < nargs; i++)
-        argv[i+1] = eval(args[i], locals, nl);
+    i = 0;
+    if (eval0) { /* 0-th argument has already been evaluated */
+        argv[1] = eval0; i++;
+    }
+    for(; i < nargs; i++) {
+        argv[i+1] = eval(args[i], locals, nl, ngensym);
+    }
     jl_value_t *result = jl_apply(f, &argv[1], nargs);
     JL_GC_POP();
     return result;
@@ -75,9 +76,9 @@ jl_value_t *jl_eval_global_var(jl_module_t *m, jl_sym_t *e)
     return v;
 }
 
-void jl_check_static_parameter_conflicts(jl_lambda_info_t *li, jl_tuple_t *t, jl_sym_t *fname);
+void jl_check_static_parameter_conflicts(jl_lambda_info_t *li, jl_svec_t *t, jl_sym_t *fname);
 
-int jl_has_intrinsics(jl_expr_t *e);
+int jl_has_intrinsics(jl_expr_t *ast, jl_expr_t *e, jl_module_t *m);
 
 extern int jl_boot_file_loaded;
 extern int inside_typedef;
@@ -87,14 +88,15 @@ static int equiv_type(jl_datatype_t *dta, jl_datatype_t *dtb)
 {
     return (jl_typeof(dta) == jl_typeof(dtb) &&
             // TODO: can't yet handle parametric types due to how constructors work
-            dta->parameters == jl_null &&
+            dta->parameters == jl_emptysvec &&
             dta->name->name == dtb->name->name &&
             jl_egal((jl_value_t*)dta->types, (jl_value_t*)dtb->types) &&
             dta->abstract == dtb->abstract &&
             dta->mutabl == dtb->mutabl &&
             dta->size == dtb->size &&
+            dta->ninitialized == dtb->ninitialized &&
             jl_egal((jl_value_t*)dta->super, (jl_value_t*)dtb->super) &&
-            jl_egal((jl_value_t*)dta->names, (jl_value_t*)dtb->names) &&
+            jl_egal((jl_value_t*)dta->name->names, (jl_value_t*)dtb->name->names) &&
             jl_egal((jl_value_t*)dta->parameters, (jl_value_t*)dtb->parameters));
 }
 
@@ -104,7 +106,23 @@ static void check_can_assign_type(jl_binding_t *b)
         jl_errorf("invalid redefinition of constant %s", b->name->name);
 }
 
-static jl_value_t *eval(jl_value_t *e, jl_value_t **locals, size_t nl)
+void jl_reinstantiate_inner_types(jl_datatype_t *t);
+
+void jl_set_datatype_super(jl_datatype_t *tt, jl_value_t *super)
+{
+    if (!jl_is_datatype(super) || !jl_is_abstracttype(super) ||
+        tt->name == ((jl_datatype_t*)super)->name ||
+        jl_subtype(super,(jl_value_t*)jl_vararg_type,0) ||
+        jl_is_tuple_type(super) ||
+        jl_subtype(super,(jl_value_t*)jl_type_type,0)) {
+        jl_errorf("invalid subtyping in definition of %s",
+                  jl_symbol_name(tt->name->name));
+    }
+    tt->super = (jl_datatype_t*)super;
+    jl_gc_wb(tt, tt->super);
+}
+
+static jl_value_t *eval(jl_value_t *e, jl_value_t **locals, size_t nl, size_t ngensym)
 {
     if (jl_is_symbol(e)) {
         jl_value_t *v;
@@ -124,7 +142,14 @@ static jl_value_t *eval(jl_value_t *e, jl_value_t **locals, size_t nl)
         return v;
     }
     if (jl_is_symbolnode(e)) {
-        return eval((jl_value_t*)jl_symbolnode_sym(e), locals, nl);
+        return eval((jl_value_t*)jl_symbolnode_sym(e), locals, nl, ngensym);
+    }
+    if (jl_is_gensym(e)) {
+        ssize_t genid = ((jl_gensym_t*)e)->id;
+        if (genid >= ngensym || genid < 0)
+            jl_error("access to invalid GenSym location");
+        else
+            return locals[nl*2 + genid];
     }
     if (jl_is_quotenode(e)) {
         return jl_fieldref(e,0);
@@ -137,23 +162,24 @@ static jl_value_t *eval(jl_value_t *e, jl_value_t **locals, size_t nl)
         return v;
     }
     if (!jl_is_expr(e)) {
-        if (jl_is_getfieldnode(e)) {
-            jl_value_t *v = eval(jl_getfieldnode_val(e), locals, nl);
-            jl_value_t *gfargs[2] = {v, (jl_value_t*)jl_getfieldnode_name(e)};
+        if (jl_is_globalref(e)) {
+            jl_value_t *gfargs[2] = {(jl_value_t*)jl_globalref_mod(e), (jl_value_t*)jl_globalref_name(e)};
             return jl_f_get_field(NULL, gfargs, 2);
         }
         if (jl_is_lambda_info(e)) {
             jl_lambda_info_t *li = (jl_lambda_info_t*)e;
             if (jl_boot_file_loaded && li->ast && jl_is_expr(li->ast)) {
                 li->ast = jl_compress_ast(li, li->ast);
+                jl_gc_wb(li, li->ast);
             }
-            return (jl_value_t*)jl_new_closure(NULL, (jl_value_t*)jl_null, li);
+            return (jl_value_t*)jl_new_closure(NULL, (jl_value_t*)jl_emptysvec, li);
         }
         if (jl_is_linenode(e)) {
             jl_lineno = jl_linenode_line(e);
         }
         if (jl_is_newvarnode(e)) {
             jl_value_t *var = jl_fieldref(e,0);
+            assert(!jl_is_gensym(var));
             assert(jl_is_symbol(var));
             for(size_t i=0; i < nl; i++) {
                 if (locals[i*2] == var) {
@@ -166,14 +192,14 @@ static jl_value_t *eval(jl_value_t *e, jl_value_t **locals, size_t nl)
         return e;
     }
     jl_expr_t *ex = (jl_expr_t*)e;
-    jl_value_t **args = &jl_cellref(ex->args,0);
+    jl_value_t **args = (jl_value_t**)jl_array_data(ex->args);
     size_t nargs = jl_array_len(ex->args);
-    if (ex->head == call_sym ||  ex->head == call1_sym) {
+    if (ex->head == call_sym) {
         if (jl_is_lambda_info(args[0])) {
             // directly calling an inner function ("let")
             jl_lambda_info_t *li = (jl_lambda_info_t*)args[0];
             if (jl_is_expr(li->ast) && !jl_lam_vars_captured((jl_expr_t*)li->ast) &&
-                !jl_has_intrinsics((jl_expr_t*)li->ast)) {
+                !jl_has_intrinsics((jl_expr_t*)li->ast, (jl_expr_t*)li->ast, jl_current_module)) {
                 size_t na = nargs-1;
                 if (na == 0)
                     return jl_interpret_toplevel_thunk(li);
@@ -182,17 +208,16 @@ static jl_value_t *eval(jl_value_t *e, jl_value_t **locals, size_t nl)
                 if (nreq==0 || !jl_is_rest_arg(jl_cellref(formals,nreq-1))) {
                     jl_value_t **ar;
                     JL_GC_PUSHARGS(ar, na*2);
-                    for(int i=0; i < na*2; i++) {
-                        ar[i] = NULL;
-                    }
                     for(int i=0; i < na; i++) {
-                        ar[i*2+1] = eval(args[i+1], locals, nl);
+                        ar[i*2+1] = eval(args[i+1], locals, nl, ngensym);
+                        jl_gc_wb(ex->args, ar[i*2+1]);
                     }
                     if (na != nreq) {
                         jl_error("wrong number of arguments");
                     }
                     for(int i=0; i < na; i++) {
-                        ar[i*2] = (jl_value_t*)jl_decl_var(jl_cellref(formals,i));
+                        jl_value_t *v = jl_cellref(formals, i);
+                        ar[i*2] = (jl_is_gensym(v)) ? v : (jl_value_t*)jl_decl_var(v);
                     }
                     jl_value_t *ret = jl_interpret_toplevel_thunk_with(li, ar, na);
                     JL_GC_POP();
@@ -200,34 +225,51 @@ static jl_value_t *eval(jl_value_t *e, jl_value_t **locals, size_t nl)
                 }
             }
         }
-        jl_function_t *f = (jl_function_t*)eval(args[0], locals, nl);
-        if (!jl_is_func(f))
-            jl_type_error("apply", (jl_value_t*)jl_function_type,
-                          (jl_value_t*)f);
-        return do_call(f, &args[1], nargs-1, locals, nl);
+        jl_function_t *f = (jl_function_t*)eval(args[0], locals, nl, ngensym);
+        if (jl_is_func(f))
+            return do_call(f, &args[1], nargs-1, NULL, locals, nl, ngensym);
+        else
+            return do_call(jl_module_call_func(jl_current_module), args, nargs, (jl_value_t*)f, locals, nl, ngensym);
     }
     else if (ex->head == assign_sym) {
         jl_value_t *sym = args[0];
-        assert(jl_is_symbol(sym));
-        size_t i;
-        for (i=0; i < nl; i++) {
-            if (locals[i*2] == sym) {
-                return (locals[i*2+1] = eval(args[1], locals, nl));
+        jl_value_t *rhs = eval(args[1], locals, nl, ngensym);
+        if (jl_is_gensym(sym)) {
+            ssize_t genid = ((jl_gensym_t*)sym)->id;
+            if (genid >= ngensym || genid < 0)
+                jl_error("assignment to invalid GenSym location");
+            locals[nl*2 + genid] = rhs;
+            return rhs;
+        }
+        if (jl_is_symbol(sym)) {
+            size_t i;
+            for (i=0; i < nl; i++) {
+                if (locals[i*2] == sym) {
+                    locals[i*2+1] = rhs;
+                    return rhs;
+                }
             }
         }
-        jl_binding_t *b = jl_get_binding_wr(jl_current_module, (jl_sym_t*)sym);
-        jl_value_t *rhs = eval(args[1], locals, nl);
+        jl_module_t *m = jl_current_module;
+        if (jl_is_globalref(sym)) {
+            m = jl_globalref_mod(sym);
+            sym = (jl_value_t*)jl_globalref_name(sym);
+        }
+        assert(jl_is_symbol(sym));
+        JL_GC_PUSH1(&rhs);
+        jl_binding_t *b = jl_get_binding_wr(m, (jl_sym_t*)sym);
         jl_checked_assignment(b, rhs);
+        JL_GC_POP();
         return rhs;
     }
     else if (ex->head == new_sym) {
-        jl_value_t *thetype = eval(args[0], locals, nl);
+        jl_value_t *thetype = eval(args[0], locals, nl, ngensym);
         jl_value_t *v=NULL;
         JL_GC_PUSH2(&thetype, &v);
         assert(jl_is_structtype(thetype));
         v = jl_new_struct_uninit((jl_datatype_t*)thetype);
         for(size_t i=1; i < nargs; i++) {
-            jl_set_nth_field(v, i-1, eval(args[i], locals, nl));
+            jl_set_nth_field(v, i-1, eval(args[i], locals, nl, ngensym));
         }
         JL_GC_POP();
         return v;
@@ -236,7 +278,7 @@ static jl_value_t *eval(jl_value_t *e, jl_value_t **locals, size_t nl)
         return (jl_value_t*)jl_nothing;
     }
     else if (ex->head == body_sym) {
-        return eval_body(ex->args, locals, nl, 0, 0);
+        return eval_body(ex->args, locals, nl, ngensym, 0, 0);
     }
     else if (ex->head == exc_sym) {
         return jl_exception_in_transit;
@@ -247,23 +289,19 @@ static jl_value_t *eval(jl_value_t *e, jl_value_t **locals, size_t nl)
     else if (ex->head == method_sym) {
         jl_sym_t *fname = (jl_sym_t*)args[0];
         jl_value_t **bp=NULL;
+        jl_value_t *bp_owner=NULL;
         jl_binding_t *b=NULL;
         jl_value_t *gf=NULL;
         int kw=0;
-        if (jl_is_expr(fname)) {
-            if (((jl_expr_t*)fname)->head == kw_sym) {
+        if (jl_is_expr(fname) || jl_is_globalref(fname)) {
+            if (jl_is_expr(fname) && ((jl_expr_t*)fname)->head == kw_sym) {
                 kw = 1;
                 fname = (jl_sym_t*)jl_exprarg(fname, 0);
             }
-            gf = eval((jl_value_t*)fname, locals, nl);
-            assert(jl_is_function(gf));
-            if (!jl_is_gf(gf)) jl_error("cannot redefine non-generic function");
+            gf = eval((jl_value_t*)fname, locals, nl, ngensym);
             if (jl_is_expr(fname))
                 fname = (jl_sym_t*)jl_fieldref(jl_exprarg(fname, 2), 0);
-            if (!kw)
-                bp = &gf;
-            else
-                bp = (jl_value_t**)&((jl_methtable_t*)((jl_function_t*)gf)->env)->kwsorter;
+            bp = &gf;
             assert(jl_is_symbol(fname));
         }
         else {
@@ -276,21 +314,24 @@ static jl_value_t *eval(jl_value_t *e, jl_value_t **locals, size_t nl)
             if (bp == NULL) {
                 b = jl_get_binding_for_method_def(jl_current_module, fname);
                 bp = &b->value;
+                bp_owner = (jl_value_t*)jl_current_module;
             }
         }
+        if (jl_expr_nargs(ex) == 1)
+            return jl_generic_function_def(fname, bp, bp_owner, b);
         jl_value_t *atypes=NULL, *meth=NULL;
         JL_GC_PUSH2(&atypes, &meth);
-        atypes = eval(args[1], locals, nl);
+        atypes = eval(args[1], locals, nl, ngensym);
         if (jl_is_lambda_info(args[2])) {
-            jl_check_static_parameter_conflicts((jl_lambda_info_t*)args[2], (jl_tuple_t*)jl_t1(atypes), fname);
+            jl_check_static_parameter_conflicts((jl_lambda_info_t*)args[2], (jl_svec_t*)jl_svecref(atypes,1), fname);
         }
-        meth = eval(args[2], locals, nl);
-        jl_method_def(fname, bp, b, (jl_tuple_t*)atypes, (jl_function_t*)meth);
+        meth = eval(args[2], locals, nl, ngensym);
+        jl_method_def(fname, bp, bp_owner, b, (jl_svec_t*)atypes, (jl_function_t*)meth, args[3], NULL, kw);
         JL_GC_POP();
         return *bp;
     }
     else if (ex->head == copyast_sym) {
-        return jl_copy_ast(eval(args[0], locals, nl));
+        return jl_copy_ast(eval(args[0], locals, nl, ngensym));
     }
     else if (ex->head == const_sym) {
         jl_value_t *sym = args[0];
@@ -315,20 +356,22 @@ static jl_value_t *eval(jl_value_t *e, jl_value_t **locals, size_t nl)
     }
     else if (ex->head == abstracttype_sym) {
         jl_value_t *name = args[0];
-        jl_value_t *para = eval(args[1], locals, nl);
+        jl_value_t *para = eval(args[1], locals, nl, ngensym);
         jl_value_t *super = NULL;
         jl_value_t *temp = NULL;
-        JL_GC_PUSH3(&para, &super, &temp);
-        assert(jl_is_tuple(para));
+        jl_datatype_t *dt = NULL;
+        JL_GC_PUSH4(&para, &super, &temp, &dt);
+        assert(jl_is_svec(para));
         assert(jl_is_symbol(name));
-        jl_datatype_t *dt =
-            jl_new_abstracttype(name, jl_any_type, (jl_tuple_t*)para);
+        dt = jl_new_abstracttype(name, jl_any_type, (jl_svec_t*)para);
         jl_binding_t *b = jl_get_binding_wr(jl_current_module, (jl_sym_t*)name);
         temp = b->value;
         check_can_assign_type(b);
         b->value = (jl_value_t*)dt;
-        super = eval(args[2], locals, nl);
+        jl_gc_wb_binding(b, dt);
+        super = eval(args[2], locals, nl, ngensym);
         jl_set_datatype_super(dt, super);
+        jl_reinstantiate_inner_types(dt);
         b->value = temp;
         if (temp==NULL || !equiv_type(dt, (jl_datatype_t*)temp)) {
             jl_checked_assignment(b, (jl_value_t*)dt);
@@ -339,25 +382,27 @@ static jl_value_t *eval(jl_value_t *e, jl_value_t **locals, size_t nl)
     else if (ex->head == bitstype_sym) {
         jl_value_t *name = args[0];
         jl_value_t *super = NULL, *para = NULL, *vnb = NULL, *temp = NULL;
-        JL_GC_PUSH3(&para, &super, &temp);
+        jl_datatype_t *dt = NULL;
+        JL_GC_PUSH4(&para, &super, &temp, &dt);
         assert(jl_is_symbol(name));
-        para = eval(args[1], locals, nl);
-        assert(jl_is_tuple(para));
-        vnb  = eval(args[2], locals, nl);
+        para = eval(args[1], locals, nl, ngensym);
+        assert(jl_is_svec(para));
+        vnb  = eval(args[2], locals, nl, ngensym);
         if (!jl_is_long(vnb))
             jl_errorf("invalid declaration of bits type %s", ((jl_sym_t*)name)->name);
-        int32_t nb = jl_unbox_long(vnb);
+        ssize_t nb = jl_unbox_long(vnb);
         if (nb < 1 || nb>=(1<<23) || (nb&7) != 0)
             jl_errorf("invalid number of bits in type %s",
                       ((jl_sym_t*)name)->name);
-        jl_datatype_t *dt =
-            jl_new_bitstype(name, jl_any_type, (jl_tuple_t*)para, nb);
+        dt = jl_new_bitstype(name, jl_any_type, (jl_svec_t*)para, nb);
         jl_binding_t *b = jl_get_binding_wr(jl_current_module, (jl_sym_t*)name);
         temp = b->value;
         check_can_assign_type(b);
         b->value = (jl_value_t*)dt;
-        super = eval(args[3], locals, nl);
+        jl_gc_wb_binding(b, dt);
+        super = eval(args[3], locals, nl, ngensym);
         jl_set_datatype_super(dt, super);
+        jl_reinstantiate_inner_types(dt);
         b->value = temp;
         if (temp==NULL || !equiv_type(dt, (jl_datatype_t*)temp)) {
             jl_checked_assignment(b, (jl_value_t*)dt);
@@ -366,54 +411,60 @@ static jl_value_t *eval(jl_value_t *e, jl_value_t **locals, size_t nl)
         return (jl_value_t*)jl_nothing;
     }
     else if (ex->head == compositetype_sym) {
-        void jl_add_constructors(jl_datatype_t *t);
         jl_value_t *name = args[0];
         assert(jl_is_symbol(name));
-        jl_value_t *para = eval(args[1], locals, nl);
-        assert(jl_is_tuple(para));
+        jl_value_t *para = eval(args[1], locals, nl, ngensym);
+        assert(jl_is_svec(para));
         jl_value_t *temp = NULL;
         jl_value_t *super = NULL;
         jl_datatype_t *dt = NULL;
         JL_GC_PUSH4(&para, &super, &temp, &dt);
-        temp = eval(args[2], locals, nl);  // field names
-        dt = jl_new_datatype((jl_sym_t*)name, jl_any_type, (jl_tuple_t*)para,
-                             (jl_tuple_t*)temp, NULL,
-                             0, args[6]==jl_true ? 1 : 0);
-        dt->fptr = jl_f_ctor_trampoline;
-        dt->ctor_factory = eval(args[3], locals, nl);
+        temp = eval(args[2], locals, nl, ngensym);  // field names
+        dt = jl_new_datatype((jl_sym_t*)name, jl_any_type, (jl_svec_t*)para,
+                             (jl_svec_t*)temp, NULL,
+                             0, args[5]==jl_true ? 1 : 0, jl_unbox_long(args[6]));
 
         jl_binding_t *b = jl_get_binding_wr(jl_current_module, (jl_sym_t*)name);
         temp = b->value;  // save old value
         // temporarily assign so binding is available for field types
         check_can_assign_type(b);
         b->value = (jl_value_t*)dt;
+        jl_gc_wb_binding(b,dt);
 
         JL_TRY {
+            super = eval(args[3], locals, nl, ngensym);
+            jl_set_datatype_super(dt, super);
             // operations that can fail
             inside_typedef = 1;
-            dt->types = (jl_tuple_t*)eval(args[5], locals, nl);
+            dt->types = (jl_svec_t*)eval(args[4], locals, nl, ngensym);
+            jl_gc_wb(dt, dt->types);
             inside_typedef = 0;
-            jl_check_type_tuple(dt->types, dt->name->name, "type definition");
-            super = eval(args[4], locals, nl);
-            jl_set_datatype_super(dt, super);
+            for(size_t i=0; i < jl_svec_len(dt->types); i++) {
+                jl_value_t *elt = jl_svecref(dt->types, i);
+                if (!jl_is_type(elt) && !jl_is_typevar(elt))
+                    jl_type_error_rt(dt->name->name->name, "type definition", (jl_value_t*)jl_type_type, elt);
+            }
+            jl_reinstantiate_inner_types(dt);
         }
         JL_CATCH {
             b->value = temp;
             jl_rethrow();
         }
-        for(size_t i=0; i < jl_tuple_len(para); i++) {
-            ((jl_tvar_t*)jl_tupleref(para,i))->bound = 0;
+        for(size_t i=0; i < jl_svec_len(para); i++) {
+            ((jl_tvar_t*)jl_svecref(para,i))->bound = 0;
         }
         jl_compute_field_offsets(dt);
+        if (para == (jl_value_t*)jl_emptysvec && jl_is_datatype_singleton(dt)) {
+            dt->instance = newstruct(dt);
+            jl_gc_wb(dt, dt->instance);
+        }
 
         b->value = temp;
         if (temp==NULL || !equiv_type(dt, (jl_datatype_t*)temp)) {
             jl_checked_assignment(b, (jl_value_t*)dt);
-
-            jl_add_constructors(dt);
         }
         else {
-            // TODO: remove all old ctors and set temp->ctor_factory = dt->ctor_factory
+            // TODO: remove all old ctors and set temp->name->ctor_factory = dt->name->ctor_factory
         }
 
         JL_GC_POP();
@@ -422,13 +473,14 @@ static jl_value_t *eval(jl_value_t *e, jl_value_t **locals, size_t nl)
     else if (ex->head == macro_sym) {
         jl_sym_t *nm = (jl_sym_t*)args[0];
         assert(jl_is_symbol(nm));
-        jl_function_t *f = (jl_function_t*)eval(args[1], locals, nl);
+        jl_function_t *f = (jl_function_t*)eval(args[1], locals, nl, ngensym);
         JL_GC_PUSH1(&f);
         assert(jl_is_function(f));
         if (jl_boot_file_loaded &&
             f->linfo && f->linfo->ast && jl_is_expr(f->linfo->ast)) {
             jl_lambda_info_t *li = f->linfo;
             li->ast = jl_compress_ast(li, li->ast);
+            jl_gc_wb(li, li->ast);
             li->name = nm;
         }
         jl_set_global(jl_current_module, nm, (jl_value_t*)f);
@@ -452,17 +504,22 @@ static jl_value_t *eval(jl_value_t *e, jl_value_t **locals, size_t nl)
     else if (ex->head == boundscheck_sym) {
         return (jl_value_t*)jl_nothing;
     }
+    else if (ex->head == fastmath_sym) {
+        return (jl_value_t*)jl_nothing;
+    }
     else if (ex->head == simdloop_sym) {
         return (jl_value_t*)jl_nothing;
     }
+    else if (ex->head == meta_sym) {
+        return (jl_value_t*)jl_nothing;
+    }
     jl_errorf("unsupported or misplaced expression %s", ex->head->name);
     return (jl_value_t*)jl_nothing;
 }
 
-static int label_idx(jl_value_t *tgt, jl_array_t *stmts)
+static int label_idx(long ltgt, jl_array_t *stmts)
 {
     size_t j;
-    long ltgt = jl_unbox_long(tgt);
     for(j=0; j < stmts->nrows; j++) {
         jl_value_t *l = jl_cellref(stmts,j);
         if (jl_is_labelnode(l) && jl_labelnode_label(l)==ltgt)
@@ -474,10 +531,24 @@ static int label_idx(jl_value_t *tgt, jl_array_t *stmts)
 
 jl_value_t *jl_toplevel_eval_body(jl_array_t *stmts)
 {
-    return eval_body(stmts, NULL, 0, 0, 1);
+    ssize_t ngensym = 0;
+    size_t i, l = jl_array_len(stmts);
+    for (i = 0; i < l; i++) {
+        ssize_t maxid = jl_max_jlgensym_in(jl_cellref(stmts, i))+1;
+        if (maxid > ngensym)
+            ngensym = maxid;
+    }
+    jl_value_t **locals = NULL;
+    if (ngensym > 0) {
+        JL_GC_PUSHARGS(locals, ngensym);
+    }
+    jl_value_t *ret = eval_body(stmts, locals, 0, ngensym, 0, 1);
+    if (ngensym > 0)
+        JL_GC_POP();
+    return ret;
 }
 
-static jl_value_t *eval_body(jl_array_t *stmts, jl_value_t **locals, size_t nl,
+static jl_value_t *eval_body(jl_array_t *stmts, jl_value_t **locals, size_t nl, size_t ngensym,
                              int start, int toplevel)
 {
     jl_handler_t __eh;
@@ -486,15 +557,15 @@ static jl_value_t *eval_body(jl_array_t *stmts, jl_value_t **locals, size_t nl,
     while (1) {
         jl_value_t *stmt = jl_cellref(stmts,i);
         if (jl_is_gotonode(stmt)) {
-            i = label_idx(jl_fieldref(stmt,0), stmts);
+            i = label_idx(jl_gotonode_label(stmt), stmts);
             continue;
         }
         if (jl_is_expr(stmt)) {
             jl_sym_t *head = ((jl_expr_t*)stmt)->head;
             if (head == goto_ifnot_sym) {
-                jl_value_t *cond = eval(jl_exprarg(stmt,0), locals, nl);
+                jl_value_t *cond = eval(jl_exprarg(stmt,0), locals, nl, ngensym);
                 if (cond == jl_false) {
-                    i = label_idx(jl_exprarg(stmt,1), stmts);
+                    i = label_idx(jl_unbox_long(jl_exprarg(stmt, 1)), stmts);
                     continue;
                 }
                 else if (cond != jl_true) {
@@ -507,19 +578,19 @@ static jl_value_t *eval_body(jl_array_t *stmts, jl_value_t **locals, size_t nl,
                 if (toplevel && jl_is_toplevel_only_expr(ex))
                     return jl_toplevel_eval(ex);
                 else
-                    return eval(ex, locals, nl);
+                    return eval(ex, locals, nl, ngensym);
             }
             else if (head == enter_sym) {
                 jl_enter_handler(&__eh);
                 if (!jl_setjmp(__eh.eh_ctx,1)) {
-                    return eval_body(stmts, locals, nl, i+1, toplevel);
+                    return eval_body(stmts, locals, nl, ngensym, i+1, toplevel);
                 }
                 else {
 #ifdef _OS_WINDOWS_
                     if (jl_exception_in_transit == jl_stackovf_exception)
                         _resetstkoflw();
 #endif
-                    i = label_idx(jl_exprarg(stmt,0), stmts);
+                    i = label_idx(jl_unbox_long(jl_exprarg(stmt,0)), stmts);
                     continue;
                 }
             }
@@ -531,14 +602,14 @@ static jl_value_t *eval_body(jl_array_t *stmts, jl_value_t **locals, size_t nl,
                 if (toplevel && jl_is_toplevel_only_expr(stmt))
                     jl_toplevel_eval(stmt);
                 else
-                    eval(stmt, locals, nl);
+                    eval(stmt, locals, nl, ngensym);
             }
         }
         else {
             if (toplevel && jl_is_toplevel_only_expr(stmt))
                 jl_toplevel_eval(stmt);
             else
-                eval(stmt, locals, nl);
+                eval(stmt, locals, nl, ngensym);
         }
         i++;
     }
@@ -551,23 +622,25 @@ jl_value_t *jl_interpret_toplevel_thunk_with(jl_lambda_info_t *lam,
 {
     jl_expr_t *ast = (jl_expr_t*)lam->ast;
     jl_array_t *stmts = jl_lam_body(ast)->args;
-    jl_array_t *l = jl_lam_locals(ast);
-    size_t llength = jl_array_len(l);
-    jl_value_t **names = &((jl_value_t**)l->data)[0];
+    size_t nargs = jl_array_len(jl_lam_args(ast));
+    jl_array_t *l = jl_lam_vinfo(ast);
+    size_t llength = jl_array_len(l) - nargs;
     nl += llength;
     jl_value_t **locals;
-    JL_GC_PUSHARGS(locals, nl*2);
-    jl_value_t *r = (jl_value_t*)jl_null;
+    jl_value_t *gensym_types = jl_lam_gensyms(ast);
+    size_t ngensym = (jl_is_array(gensym_types) ? jl_array_len(gensym_types) : jl_unbox_gensym(gensym_types));
+    JL_GC_PUSHARGS(locals, nl*2+ngensym);
+    jl_value_t *r = (jl_value_t*)jl_nothing;
     size_t i=0;
     for(i=0; i < llength; i++) {
-        locals[i*2]   = names[i];
-        locals[i*2+1] = NULL;
+        locals[i*2]   = jl_cellref(jl_cellref(l,i+nargs),0);
+        //locals[i*2+1] = NULL;   // init'd by JL_GC_PUSHARGS
     }
     for(; i < nl; i++) {
         locals[i*2]   = loc[(i-llength)*2];
         locals[i*2+1] = loc[(i-llength)*2+1];
     }
-    r = eval_body(stmts, locals, nl, 0, 1);
+    r = eval_body(stmts, locals, nl, ngensym, 0, 1);
     JL_GC_POP();
     return r;
 }
