@@ -3541,3 +3541,19 @@ let
     arrayset_unknown_dim(Int, 2)
     arrayset_unknown_dim(Int, 3)
 end
+
+# issue #15848
+@test_throws UndefVarError let x
+    foo() = x
+    foo()
+end
+
+# issue discovered in #11973
+for j = 1:1
+    x = try
+        error()
+        2
+    catch
+        continue
+    end
+end
