@@ -94,3 +94,16 @@ x.nzval[1] = 0
 let a = QuoteNode(1), b = QuoteNode(1.0)
     @test (hash(a)==hash(b)) == (a==b)
 end
+
+# issue 15659
+for prec in [3, 11, 15, 16, 31, 32, 33, 63, 64, 65, 254, 255, 256, 257, 258, 1023, 1024, 1025],
+    v in Any[-0.0, 0, 1, -1, 1//10, 2//10, 3//10, 1//2, pi]
+    with_bigfloat_precision(prec) do
+        x = convert(BigFloat, v)
+        @test precision(x) == prec
+        num, pow, den = Base.decompose(x)
+        y = num*big(2.0)^pow/den
+        @test precision(y) == prec
+        @test isequal(x, y)
+    end
+end
