@@ -333,3 +333,11 @@ let cmd = AbstractString[]
 end
 
 @windows_only ENV["PATH"] = oldpath
+
+# Test for PR 17803
+let p=Pipe()
+    Base.link_pipe(p; julia_only_read=true, julia_only_write=true)
+    ccall(:jl_static_show, Void, (Ptr{Void}, Any), p.in, Int128(-1))
+    @async close(p.in)
+    @test readall(p.out) == "Int128(0xffffffffffffffffffffffffffffffff)"
+end
